<div class="form-group {{ item.css_classes }}">
    <label for="{{ item.id_for_label }}" class="control-label col-md-3">
        {{ item.label }}
    </label>
    <div class="col-md-9{% if item.errors %} alert alert-danger{% endif %}">
        {{ item }}
        {% for error in item.errors %}
            <div>{{ error }}</div>
        {% endfor %}

        {% if item.help_text %}
            <span class="help-block">{{ item.help_text | safe }}</span>
        {% endif %}
    </div>
</div>
